function [Y] = thsp(DATA,R,S)
// 
// Algorithm Homogeneity Technique
// Developed by: Junio Moreira / Roberto Alves Braga Junior
// Federal University of Lavras - UFLA
// =================================================
//
// In:  Cardoso, R.R. ; Braga, R.A. ; Rabal, H.J. Alternative protocols on 
//      dynamic speckle laser analysis. SPIE 8413, V International Conference 
//      on Speckle Metrology. 2012
//
//  Output:
//   Y is the Constrast image
//   X is the value of the contrast
//
// =================================================
// Main Program
//==================================================
// In Scilab 5.4 you have to install the libraries SPD, SIVP, PLOTLIB
// BSLD (Library Speckle Laser Dynamic) version 1.0
// Date: september of 2013
// Review: october of 2013
// Software Register in Natinal Institute of Industrial Property INPI
// Registration number “Bibliotecas Speckle Laser Dinâmico para Scilab e Matlab” BR 51 2014 000018 4
// Type of Registration: Software free to use (Library free to use)
// This library is totally free to use in education, research and other noncommercial activities.
//
// If you use it, please, cite it properly:
// Moreira, J, Cardoso, RR, Braga, RA 2014 Quality test protocol to dynamic laser speckle analysis. 
// Optics and Lasers in Engineering. 10.1016/j.optlaseng.2014.04.005
//
// The homegeneity adopts some functions that follow
// =================================================
// Time History Speckle Patterns - THPS
//  Input:
//  DATA is the speckle data
//  R is the parameter of analysis: 
//            if R is equal 1, you choose the line of images to make THSP
//            and if R is equal 2 you choose the columns of images to make THSP
//            and if R is equal 3 you choose the thsp randon            
//  S is the line, column or size used to make the time history speckle patterns
//
//  Output:
//  Y is the time history speckle patterns image
//
// =============================================================================
// Begin
//
    a = size(DATA);        
    
    if R == 1 then
        Y = zeros(a(1,2),a(1,3));
        for b = 1:a(1,3)
            Y(:,b) = DATA(S,:,b)';
        end        
    elseif R == 2 then
        Y = zeros(a(1,1),a(1,3));
        for b = 1:a(1,3)
            Y(:,b) = DATA(:,S,b);
        end                
    elseif R == 3        
        matriz_ists = zeros (2,a(1,2));
        Y = zeros(S,a(1,3));        
        for b = 1:S     
            ists1 = ceil((rand(1,1))*a(1,1));
            ists2 = ceil((rand(1,1))*a(1,2));            
            matriz_ists (1,b) = ists1;
            matriz_ists (2,b) = ists2;
            for c = 1:a(1,3)                                
                Y(b,c) = DATA(ists1,ists2,c);
            end            
        end                  
    else
        error('Error in the THSP image')
    end
    
endfunction
// =============================================================================

function [Y] = moc(DATA)
//
// Cooccurence Matrix Analysis - MOC
// In: ARIZAGA, R. et. al. Speckle time evolution characterization by the 
//     co-occurence matrix analysis. Optics and Laser Technology, Amsterdam, 
//     v. 31, n. 2, p. 163-169, 1999.
//
//  Input:
//  DATA is the THSP data
//
//  Output:
//  Y is the Occurrence Matrix Construction
//
// =============================================================================
// Begin
//      
    a = size(DATA);        
    
    Y = zeros(256,256);   
    for b = 1:a(1,1)
        for c = 1:a(1,2)-1            
            d = ((DATA(b,c))+1);
            e = ((DATA(b,c+1))+1);  
            
            if ((d >= 256) & (e >= 256))
                d = 256;
                e = 256;
            end
                                                
            Y(d,e) = Y(d,e)+1;  

         end
    end   
    
endfunction
// =============================================================================

function [Y] = avd(DATA)
//
// Absolute Value of the Differences method - AVD
//
// In: BRAGA, R.A. et al. Evaluation of activity through dynamic laser speckle 
//     using the absolute value of the differences, Optics Communications, v. 284, 
//     n. 2, p. 646-650, 2011.
//
//  Output:
//  Y are the AVD's values.
//    Y(1,1) is the value of the AVD Arizaga normalization 
//    Y(1,2) is the value of the AVD Cardoso normalization 
//  
// =============================================================================
// Begin
//    
    Y = zeros(1,2);
    a = size(DATA);    
    
    for b = 1:a(1,1)
        for c = 1:a(1,2) 
            
            norm = sum(DATA(b,:));
            if norm == 0;
                norm = 1;
            end
         
            Y(1,1) = ((DATA(b,c)*(abs(b-c)))/(norm)) + Y(1,1);
            
            Y(1,2) = ((DATA(b,c)*(abs(b-c)))/(sum(sum(DATA)))) + Y(1,2);
                        
        end
    end    
    
endfunction
// =============================================================================

function [Y] = im(DATA)
//
// Inertia Moment's - IM
// In: ARIZAGA, R. et. al. Speckle time evolution characterization by the 
//     co-occurence matrix analysis. Optics and Laser Technology, Amsterdam, 
//     v. 31, n. 2, p. 163-169, 1999.
//
// and: BRAGA, R.A. ; Cardoso, R.R. ; Bezerra, P.S. ; Wouters, F. ; Sampaio, 
//      G.R. ; Varaschin, M.S. . Biospeckle numerical values over spectral image 
//      maps of activity. Optics Communications (Print), v. 285, p. 553-561, 2012.
//
//  Output:
//  Y is the value of the Inertia Moment.
//    Y(1,1) is the value of the Inertia Moment without normalization
//    Y(1,2) is the value of the Inertia Moment with Arizaga normalization 
//    Y(1,3) is the value of the Inertia Moment with Cardoso normalization 
//
// =============================================================================
// Begin
//      
    Y = zeros(1,3);
    a = size(DATA);        
    
    for b = 1:a(1,1)
        for c = 1:a(1,2) 
            
            norm = sum(DATA(b,:));
            if norm == 0;
                norm = 1;
            end
         
            Y(1,1) = ((DATA(b,c)*((b-c)^2))/(256)) + Y(1,1);
            
            Y(1,2) = ((DATA(b,c)*((b-c)^2))/(norm)) + Y(1,2);
            
            Y(1,3) = ((DATA(b,c)*((b-c)^2))/(sum(sum(DATA)))) + Y(1,3); 
                        
        end
    end   
    
endfunction
// =============================================================================

function [X,Y] = homogeneidade(DATA,h,w)
//
//  Homogeneity test
//  In:  Cardoso, R.R. ; Braga, R.A. ; Rabal, H.J. Alternative protocols on 
//       dynamic speckle laser analysis. SPIE 8413, V International Conference 
//       on Speckle Metrology. 2012
//
//  Output:
//  Y is a matrix with the homogeneit values
//    Y(1,1) is the homogeneity value using the inertia moment method without normalization
//    Y(2,1) is the homogeneity value using the inertia moment with Arizaga normalization
//    Y(3,1) is the homogeneity value using the inertia moment with Cardoso normalization
//
//    Y(1,2) is the homogeneity value using the AVD technique with Arizaga normalization
//    Y(2,2) is the homogeneity value using the AVD technique with Cardoso normalization
//
//  X are images with the spatial variability of the homogeneity
//    X(:,:,1) is the homogeneity image using inertia moment method without normalization
//    X(:,:,2) is the homogeneity image using inertia moment method with Arizaga normalization
//    X(:,:,3) is the homogeneity image using inertia moment method with Cardoso normalization
//
//    X(:,:,4) is the homogeneity image using AVD technique with Arizaga normalization
//    X(:,:,5) is the homogeneity image using AVD technique with Cardoso normalization
//
// =============================================================================
// Begin
//
    a = size(DATA);    
    Y = zeros(3,2);
    X = zeros(a(1,1),a(1,2),5);    
    Z = zeros(h,w,a(1,3));
    H = zeros(5,((a(1,1)/h)*(a(1,2)/w)));
    i = 1;    
    
    for b = 1:h:a(1,1)-h
        for c =1:w:a(1,2)-w+1            
            Z(:,:,:) = DATA(b:b+h-1,c:c+w-1,:);
                                  
            [sts] = thsp(Z,1,w/2);            
            [ocm] = moc(sts);                
            [d] = im(ocm);
            X(b:b+h-1,c:c+w-1,1) = d(1,1);
            X(b:b+h-1,c:c+w-1,2) = d(1,2);
            X(b:b+h-1,c:c+w-1,3) = d(1,3);                  
             
            [e] = avd(ocm);
            X(b:b+h-1,c:c+w-1,4) = e(1,1);
            X(b:b+h-1,c:c+w-1,5) = e(1,2);
            H(1,i) = d(1,1);
            H(2,i) = d(1,2);
            H(3,i) = d(1,3);
            H(4,i) = e(1,1); 
            H(5,i) = e(1,2);            
            i = i+1;            
        end
    end    
    
// Homogeneity of method MI    
    Y(1,1) = 100 * (std2(H(1,:))/mean2(H(1,:)));   
    
    Y(2,1) = 100 * (std2(H(2,:))/mean2(H(2,:)));     
    
    Y(3,1) = 100 * (std2(H(3,:))/mean2(H(3,:)));     
   
// Homogeneity of method AVD       
    Y(1,2) = 100 * (std2(H(4,:))/mean2(H(4,:)));     
   
    Y(2,2) = 100 * (std2(H(5,:))/mean2(H(5,:)));   

endfunction
// =============================================================================
//
// This is the main function that you have to run
function [Y] = homogeneity(address,numImages,h,w)
//
// Algorithm Homogeneity Technique
// Developed by: Junio Moreira / Roberto Alves Braga Junior
// Federal University of Lavras - UFLA
// =================================================
//
// In:  Cardoso, R.R. ; Braga, R.A. ; Rabal, H.J. Alternative protocols on 
//      dynamic speckle laser analysis. SPIE 8413, V International Conference 
//      on Speckle Metrology. 2012
//
//  Output:
//   Y is the Constrast image
//   X is the value of the contrast
//
// =================================================
// Main Program
//==================================================
// In Scilab 5.4 you have to install the libraries SPD, SIVP, PLOTLIB
// BSLD (Library Speckle Laser Dynamic) version 1.0
// Date: september of 2013
// Review: october of 2013
// Software Register in Natinal Institute of Industrial Property INPI
// Registration number “Bibliotecas Speckle Laser Dinâmico para Scilab e Matlab” BR 51 2014 000018 4
// Type of Registration: Software free to use (Library free to use)
// This library is totally free to use in education, research and other noncommercial activities.
//
// If you use it, please, cite it properly:
// Moreira, J, Cardoso, RR, Braga, RA 2014 Quality test protocol to dynamic laser speckle analysis. 
// Optics and Lasers in Engineering. 10.1016/j.optlaseng.2014.04.005
//
//-----------------------------------------------------------------------------//
//
// Clear memory
//
    clc();
    clear all;

// Set free the memory
    stacksize('max');    

// Start Time
    initiationOfTheClock = getdate();    
    
// Set the directory of the Images    
    cd(address);           

// Identification of the images -- see the format of the image with only numbers.
    j = imread ('1.bmp');
    k = size(j);    
    line = k(1,1);
    column = k(1,2);

    DATA = zeros(line,column,numImages);    
     
// Begin 
    disp('Processing start!');
    disp('Wait...');
   
    for i = 1:numImages
        nome = [string(i)+'.bmp'];
        DATA(:,:,i) = imread(nome);        
    end    
//-----------------------------------------------------------------------------//    
//        
// Processing              
//   
    HT = 6;    
    preview = DATA(:,:,1);         
    
    clc();
    disp('Start HOMOGENEITY method.');
    disp('Keep calm, the procedure may take several minutes...')
  
    [X,Y] = homogeneidade(DATA,h,w);
    
// Directory Functions    
    //cd('C:\bsld\fsbl\');
        
// Homogeneity of method MI 
    disp('Final Results IM method: ');
    
    disp('Homogeneity of IM without normalization --> ');    
    disp(Y(1,1));
        
    disp('Homogeneity of IM with Arizaga normalization --> ');    
    disp(Y(2,1));
        
    disp('Homogeneity of IM with Cardoso normalization --> ');    
    disp(Y(3,1));
    
// Homogeneity of method AVD   
    disp('Final Results AVD method: ');    
    disp('Homogeneity of AVD with Arizaga normalization --> ');    
    disp(Y(1,2));
       
    disp('Homogeneity of AVD with Cardoso normalization --> ');    
    disp(Y(2,2));
    
//Plotting graphs
    scf(0);
    clf(0);    
    colormap gray;
    imagesc(preview(:,:));     
    axis equal;
    title('Preview Image --> ');    
    
    if (HT == 1) //Graphical output of IM without normalization
        scf(1);
        clf(1);
        imagesc(X(:,:,1));   
        axis equal;
        title('IM without normalization');
        colorbar;
    end  
    
    if (HT == 2) //Graphical output of IM with Arizaga normalization
        scf(1);
        clf(1);
        imagesc(X(:,:,2));    
        axis equal;
        title('IM with Arizaga normalization');    
        colorbar;
    end
    
    if (HT == 3) //Graphical output of IM with Cardoso normalization
        scf(1);
        clf(1);
        imagesc(X(:,:,3));    
        axis equal;
        title('IM with Cardoso normalization');    
        colorbar;
    end
    
    if (HT == 4) //Graphical output of AVD with Arizaga normalization
        scf(1);
        clf(1);
        imagesc(X(:,:,4));    
        axis equal;
        title('AVD with Arizaga normalization');    
        colorbar;
    end
    
    if (HT == 5) //Graphical output of AVD with Cardoso normalization
        scf(1);
        clf(1);
        imagesc(X(:,:,5));    
        axis equal;
        title('AVD with Cardoso normalization');    
        colorbar;
    end
    
    if (HT == 6) //Graphical output of values of IM and AVD
        scf(1);
        clf(1);
        imagesc(X(:,:,1));    
        axis equal;
        title('IM without normalization');
        colorbar;

        scf(2);
        clf(2);
        imagesc(X(:,:,2));    
        axis equal;
        title('IM with Arizaga normalization');    
        colorbar;

        scf(3);
        clf(3);
        imagesc(X(:,:,3));    
        axis equal;
        title('IM with Cardoso normalization');
        colorbar;

        scf(4);
        clf(4);
        imagesc(X(:,:,4));    
        axis equal;
        title('AVD with Arizaga normalization');  
        colorbar;

        scf(5);
        clf(5);
        imagesc(X(:,:,5));    
        axis equal;
        title('AVD with Cardoso normalization');
        colorbar;
    end   
    
// Training time
    stopOfTheClock = getdate(); //stops the clock
    etr = etime(stopOfTheClock,initiationOfTheClock); //calculates the time elapsed
    disp('');
    disp('Training time: ' + string(etr/60) + ' minutes.');    
    disp('ENDED!!!');
    sleep(5000);
    
// =============================================================================
endfunction
